#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2015, Jure Zbontar <jure.zbontar@gmail.com>
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains all functions to load and preprocess dataset
"""

import random
import math
from torch.utils import data
import h5py
import numpy as np
import cv2


# pylint: disable=too-many-instance-attributes
# -> because I see no other way to inherit from a class suffering from too many instace attributes
# pylint: disable=too-many-locals
# pylint: disable=too-many-arguments
class MiddleburyGenerator(data.Dataset):
    """
    Generate middlebury dataset
    """

    def __init__(self, file, image, cfg):
        """
        Initialization

        :param file: training or testing hdf5 file
        :param image: image hdf5 file
        :param cfg: configuration
        :type cfg: dict
        """
        self.data = None
        self.h5_file_image = h5py.File(image, "r")
        self.image = []
        self.id_image = []

        # Load the training / testing dataset
        with h5py.File(file, "r") as h5_file:
            for dst in h5_file:
                if self.data is None:
                    self.data = h5_file[dst][:]
                else:
                    self.data = np.concatenate((self.data, h5_file[dst][:]), axis=0)
                self.id_image.append(int(h5_file[dst][0, 0]))

        # Load images
        with h5py.File(image, "r") as h5_file:
            for grp in self.id_image:
                image_grp = []
                for dst in h5_file[str(int(grp))].keys():
                    image_grp.append(h5_file[str(int(grp))][dst][:])
                self.image.append(image_grp)

        # Training parameters
        self.patch_size = 11
        self.neg_low = float(cfg["dataset_neg_low"])
        self.neg_high = float(cfg["dataset_neg_high"])
        self.pos = float(cfg["dataset_pos"])

        # Data augmentation parameters
        self.transformation = cfg["data_augmentation"]
        self.scale = float(cfg["augmentation_param"]["scale"])
        self.hscale = float(cfg["augmentation_param"]["hscale"])
        self.hshear = float(cfg["augmentation_param"]["hshear"])
        self.trans = float(cfg["augmentation_param"]["trans"])
        self.rotate = float(cfg["augmentation_param"]["rotate"])
        self.brightness = float(cfg["augmentation_param"]["brightness"])
        self.contrast = float(cfg["augmentation_param"]["contrast"])
        self.d_hscale = float(cfg["augmentation_param"]["d_hscale"])
        self.d_hshear = float(cfg["augmentation_param"]["d_hshear"])
        self.d_vtrans = float(cfg["augmentation_param"]["d_vtrans"])
        self.d_rotate = float(cfg["augmentation_param"]["d_rotate"])
        self.d_brightness = float(cfg["augmentation_param"]["d_brightness"])
        self.d_contrast = float(cfg["augmentation_param"]["d_contrast"])

    def __getitem__(self, index):
        """
        Generates one sample : the left patch, the right positive patch, the right negative patch

        :return: left patch, right positive patch, right negative patch
        :rtype: np.array(3, patch_size, patch_size)
        """
        # Make patch
        id_image = int(self.data[index, 0])
        row = int(self.data[index, 1])
        col = int(self.data[index, 2])
        disp = int(self.data[index, 3])
        id_data = self.id_image.index(id_image)

        nb_illuminations = len(self.image[id_data])
        # Left illuminations
        light_l = random.randint(0, nb_illuminations - 1)

        nb_exposures = self.image[id_data][light_l].shape[0]
        # Left exposures
        exp_l = random.randint(0, nb_exposures - 1)

        # Right illuminations and exposures
        light_r = light_l
        exp_r = exp_l

        # train 20 % of the time, on images where either the shutter exposure or the arrangements of lights are
        # different for the left and right image.
        if np.random.uniform() < 0.2:
            light_r = random.randint(0, nb_illuminations - 1)

            if exp_r > self.image[id_data][light_r].shape[0] - 1:
                exp_r = random.randint(0, self.image[id_data][light_r].shape[0] - 1)

        if np.random.uniform() < 0.2:
            nb_exposures = self.image[id_data][light_r].shape[0]
            exp_r = random.randint(0, nb_exposures - 1)

        patch_radius = int(self.patch_size / 2)

        col_pos = -1
        width = self.image[id_data][light_r].shape[3] - int(self.patch_size / 2)
        # Create the positive example = x - d + pos, with pos [-pos, pos]
        while col_pos < 0 or col_pos >= width:
            col_pos = int((col - disp) + np.random.uniform(-self.pos, self.pos))

        col_neg = -1
        # Create the negative example = x - d + neg, with neg [ neg_low, neg_high]
        while col_neg < 0 or col_neg >= width:
            col_neg = int((col - disp) + np.random.uniform(self.neg_low, self.neg_high))

        if self.transformation:
            # Calculates random data augmentation
            scale_samples = np.random.uniform(self.scale, 1)
            scale = [scale_samples * np.random.uniform(self.hscale, 1), scale_samples]
            hshear = np.random.uniform(-self.hshear, self.hshear)
            trans = [np.random.uniform(-self.trans, self.trans), np.random.uniform(-self.trans, self.trans)]
            phi = np.random.uniform(-self.rotate * math.pi / 180.0, self.rotate * math.pi / 180.0)
            brightness = np.random.uniform(-self.brightness, self.brightness)
            contrast = np.random.uniform(1.0 / self.contrast, self.contrast)

            # Make the left augmented patch
            left = self.data_augmentation(
                self.image[id_data][light_l][exp_l, 0, :, :], row, col, scale, phi, trans, hshear, brightness, contrast
            )

            scale__ = [scale[0] * np.random.uniform(self.d_hscale, 1), scale[1]]
            hshear_ = hshear + np.random.uniform(-self.d_hshear, self.d_hshear)
            trans_ = [trans[0], trans[1] + np.random.uniform(-self.d_vtrans, self.d_vtrans)]
            phi_ = phi + np.random.uniform(-self.d_rotate * math.pi / 180.0, self.d_rotate * math.pi / 180.0)
            brightness_ = brightness + np.random.uniform(-self.d_brightness, self.d_brightness)
            contrast_ = contrast * np.random.uniform(1 / self.d_contrast, self.d_contrast)

            # Make the right positive augmented patch
            right_pos = self.data_augmentation(
                self.image[id_data][light_r][exp_r, 1, :, :],
                row,
                col_pos,
                scale__,
                phi_,
                trans_,
                hshear_,
                brightness_,
                contrast_,
            )

            # Make the right negative augmented patch
            right_neg = self.data_augmentation(
                self.image[id_data][light_r][exp_r, 1, :, :],
                row,
                col_neg,
                scale__,
                phi_,
                trans_,
                hshear_,
                brightness_,
                contrast_,
            )

        else:
            # Make the left patch
            left = self.image[id_data][light_l][
                exp_l, 0, row - patch_radius : row + patch_radius + 1, col - patch_radius : col + patch_radius + 1
            ]
            # Make the right positive patch
            right_pos = self.image[id_data][light_r][
                exp_r,
                1,
                row - patch_radius : row + patch_radius + 1,
                col_pos - patch_radius : patch_radius + col_pos + 1,
            ]
            # Make the right negative patch
            right_neg = self.image[id_data][light_r][
                exp_r,
                1,
                row - patch_radius : row + patch_radius + 1,
                col_neg - patch_radius : patch_radius + col_neg + 1,
            ]

        return np.stack((left, right_pos, right_neg), axis=0)

    def __len__(self):
        """
        Return the total number of samples.

        """
        return self.data.shape[0]

    def data_augmentation(self, src, row, col, scale, phi, trans, hshear, brightness, contrast):
        """
        Return augmented patch : apply affine transformations

        :param src: source image
        :param row: row center of the patch
        :param col: col center of the patch
        :param scale: scale factor
        :param phi: rotation factor
        :param trans: translation factor
        :param hshear: shear factor in horizontal direction
        :param brightness: brightness
        :param contrast: contrast
        :return: the augmented patch
        :rtype: np.array(self.patch_size, self.patch_size)
        """
        homo_matrix = np.array([[1, 0, -col], [0, 1, -row], [0, 0, 1]])
        translation_matrix = np.array([[1, 0, trans[0]], [0, 1, trans[1]], [0, 0, 1]])
        homo_matrix = np.matmul(translation_matrix, homo_matrix)

        scale_matrix = np.array([[scale[0], 0, 0], [0, scale[1], 0], [0, 0, 1]])
        homo_matrix = np.matmul(scale_matrix, homo_matrix)

        cos_phi = math.cos(phi)
        sin_phi = math.sin(phi)
        rotate_matrix = np.array([[cos_phi, sin_phi, 0], [-sin_phi, cos_phi, 0], [0, 0, 1]])
        homo_matrix = np.matmul(rotate_matrix, homo_matrix)

        shear_matrix = np.array([[1, hshear, 0], [0, 1, 0], [0, 0, 1]])
        homo_matrix = np.matmul(shear_matrix, homo_matrix)

        translation_matrix = np.array([[1, 0, (self.patch_size - 1) / 2], [0, 1, (self.patch_size - 1) / 2]])
        homo_matrix = np.matmul(translation_matrix, homo_matrix)

        dst = cv2.warpAffine(src, homo_matrix, (self.patch_size, self.patch_size))
        dst *= contrast
        dst += brightness
        return dst



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2015, Jure Zbontar <jure.zbontar@gmail.com>
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains all functions to load and preprocess dataset
"""

import math
from torch.utils import data
import numpy as np
import h5py
import cv2


# pylint: disable=too-many-instance-attributes, too-many-arguments
class DataFusionContestGenerator(data.Dataset):
    """
    Generate data fusion dataset for training
    """

    def __init__(self, sample_hdf, image_hdf, cfg):
        """
        Initialization

        """
        self.patch_size = 11
        self.data = None
        self.image = []
        # Corespondance between the index of the image in self.image, and the image number of the hdf5 file
        self.id_image = []
        sample_file = h5py.File(sample_hdf, "r")
        image_file = h5py.File(image_hdf, "r")

        for dst in sample_file:
            if self.data is None:
                self.data = sample_file[dst][:]
            else:
                self.data = np.concatenate((self.data, sample_file[dst][:]), axis=0)
            self.image.append(image_file[dst][:])
            self.id_image.append(int(sample_file[dst][0, 0]))

        self.neg_low = float(cfg["dataset_neg_low"])
        self.neg_high = float(cfg["dataset_neg_high"])
        self.pos = float(cfg["dataset_pos"])
        self.disp_vert = float(cfg["vertical_disp"])
        self.transformation = cfg["data_augmentation"]
        self.scale = float(cfg["augmentation_param"]["scale"])
        self.hscale = float(cfg["augmentation_param"]["hscale"])
        self.hshear = float(cfg["augmentation_param"]["hshear"])
        self.trans = float(cfg["augmentation_param"]["trans"])
        self.rotate = float(cfg["augmentation_param"]["rotate"])
        self.brightness = float(cfg["augmentation_param"]["brightness"])
        self.contrast = float(cfg["augmentation_param"]["contrast"])
        self.d_hscale = float(cfg["augmentation_param"]["d_hscale"])
        self.d_hshear = float(cfg["augmentation_param"]["d_hshear"])
        self.d_vtrans = float(cfg["augmentation_param"]["d_vtrans"])
        self.d_rotate = float(cfg["augmentation_param"]["d_rotate"])
        self.d_brightness = float(cfg["augmentation_param"]["d_brightness"])
        self.d_contrast = float(cfg["augmentation_param"]["d_contrast"])

    def __getitem__(self, index):
        """
        Generates one sample : the left patch, the right positive patch, the right negative patch

        :return: left patch, right positive patch, right negative patch
        :rtype: np.array(3, patch_size, patch_size)
        """
        # Make patch
        row = int(self.data[index, 1])
        col = int(self.data[index, 2])
        disp = int(self.data[index, 3])
        id_data = self.id_image.index(int(self.data[index, 0]))

        # Patch radius
        radius = int(self.patch_size / 2)
        width = self.image[id_data].shape[2] - radius

        # Column of the positive example
        x_pos = -1
        while x_pos < 0 or x_pos >= width:
            x_pos = int((col - disp) + np.random.uniform(-self.pos, self.pos))

        # Column of the negative example
        x_neg = -1
        while x_neg < 0 or x_neg >= width:
            x_neg = int((col - disp) + np.random.uniform(self.neg_low, self.neg_high))

        height = self.image[id_data].shape[1] - radius

        y_disp = -1
        while y_disp < 0 or y_disp >= height:
            y_disp = int(row + np.random.uniform(-self.disp_vert, self.disp_vert))

        if self.transformation:
            # Calculates random data augmentation
            rand_scale = np.random.uniform(self.scale, 1)
            scale = [rand_scale * np.random.uniform(self.hscale, 1), rand_scale]
            hshear = np.random.uniform(-self.hshear, self.hshear)
            trans = [np.random.uniform(-self.trans, self.trans), np.random.uniform(-self.trans, self.trans)]
            phi = np.random.uniform(-self.rotate * math.pi / 180.0, self.rotate * math.pi / 180.0)
            brightness = np.random.uniform(-self.brightness, self.brightness)
            contrast = np.random.uniform(1.0 / self.contrast, self.contrast)

            left = self.data_augmentation(
                self.image[id_data][0, :, :], row, col, scale, phi, trans, hshear, brightness, contrast
            )

            scale__ = [scale[0] * np.random.uniform(self.d_hscale, 1), scale[1]]
            hshear_ = hshear + np.random.uniform(-self.d_hshear, self.d_hshear)
            trans_ = [trans[0], trans[1] + np.random.uniform(-self.d_vtrans, self.d_vtrans)]
            phi_ = phi + np.random.uniform(-self.d_rotate * math.pi / 180.0, self.d_rotate * math.pi / 180.0)
            brightness_ = brightness + np.random.uniform(-self.d_brightness, self.d_brightness)
            contrast_ = contrast * np.random.uniform(1 / self.d_contrast, self.d_contrast)

            right_pos = self.data_augmentation(
                self.image[id_data][1, :, :], y_disp, x_pos, scale__, phi_, trans_, hshear_, brightness_, contrast_
            )

            right_neg = self.data_augmentation(
                self.image[id_data][1, :, :], y_disp, x_neg, scale__, phi_, trans_, hshear_, brightness_, contrast_
            )

        else:
            # Make the left patch
            left = self.image[id_data][0, row - radius : row + radius + 1, col - radius : col + radius + 1]
            # Make the right positive patch
            right_pos = self.image[id_data][
                1, y_disp - radius : y_disp + radius + 1, x_pos - radius : radius + x_pos + 1
            ]
            # Make the right negative patch
            right_neg = self.image[id_data][
                1, y_disp - radius : y_disp + radius + 1, x_neg - radius : radius + x_neg + 1
            ]

        return np.stack((left, right_pos, right_neg), axis=0)

    def __len__(self):
        """
        Return the total number of samples
        """
        return self.data.shape[0]

    def data_augmentation(self, src, row, col, scale, phi, trans, hshear, brightness, contrast):
        """
        Return augmented patch : apply affine transformations

        :param src: source image
        :param row: row center of the patch
        :param col: col center of the patch
        :param scale: scale factor
        :param phi: rotation factor
        :param trans: translation factor
        :param hshear: shear factor in horizontal direction
        :param brightness: brightness
        :param contrast: contrast
        :return: the augmented patch
        :rtype: np.array(self.patch_size, self.patch_size)
        """
        homo_matrix = np.array([[1, 0, -col], [0, 1, -row], [0, 0, 1]])
        translation_matrix = np.array([[1, 0, trans[0]], [0, 1, trans[1]], [0, 0, 1]])
        homo_matrix = np.matmul(translation_matrix, homo_matrix)

        scale_matrix = np.array([[scale[0], 0, 0], [0, scale[1], 0], [0, 0, 1]])
        homo_matrix = np.matmul(scale_matrix, homo_matrix)

        cos_phi = math.cos(phi)
        sin_phi = math.sin(phi)
        rotate_matrix = np.array([[cos_phi, sin_phi, 0], [-sin_phi, cos_phi, 0], [0, 0, 1]])
        homo_matrix = np.matmul(rotate_matrix, homo_matrix)

        shear_matrix = np.array([[1, hshear, 0], [0, 1, 0], [0, 0, 1]])
        homo_matrix = np.matmul(shear_matrix, homo_matrix)

        translation_matrix = np.array([[1, 0, (self.patch_size - 1) / 2], [0, 1, (self.patch_size - 1) / 2]])
        homo_matrix = np.matmul(translation_matrix, homo_matrix)

        dst = cv2.warpAffine(src, homo_matrix, (self.patch_size, self.patch_size))
        dst *= contrast
        dst += brightness
        return dst



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains all functions to generate the training and testing dataset on the Data Fusion Contest generated
with Beefrost
"""

import os
import glob
import argparse
import numpy as np
import h5py
import rasterio
from numba import njit


@njit()
def compute_mask(disp_map, mask_left, mask_right, patch_size):
    """
    Masks invalid pixels : pixel outside epipolar image

    :param disp_map: disparity map
    :type disp_map: 2D numpy array
    :param mask_left: left epipolar image mask : with the convention 0 is valid pixel in epipolar image
    :type mask_left: 2D numpy array
    :param mask_right: right epipolar image mask : with the convention 0 is valid pixel in epipolar image
    :type mask_right: 2D numpy array
    :param patch_size: patch size
    :type patch_size: int
    :return: the disparity map with invalid pixels = -9999
    :rtype: 2D numpy array
    """
    radius = int(patch_size / 2)
    nb_row, nb_col = disp_map.shape

    for row in range(radius, nb_row - radius):
        for col in range(radius, nb_col - radius):
            disp = disp_map[row, col]
            # Matching in the right image
            match = int(col + disp)

            # Negative matching for training, with maximum negative displacement for creating negative example
            neg_match = match - 6

            # If negative example is inside right epipolar image
            if radius < neg_match < (nb_col - radius) and radius < neg_match < (nb_row - radius):
                patch_left = mask_left[(row - radius) : (row + radius + 1), (col - radius) : (col + radius + 1)]
                patch_right = mask_right[(row - radius) : (row + radius + 1), (match - radius) : (match + radius + 1)]

                # Invalid patch : outside left epipolar image
                if np.sum(patch_left != 0) != 0:
                    disp_map[row, col] = -9999

                # Invalid patch : outside right epipolar image
                if np.sum(patch_right != 0) != 0:
                    disp_map[row, col] = -9999

                neg_patch_right = mask_right[
                    (row - radius) : (row + radius + 1), (neg_match - radius) : (neg_match + radius + 1)
                ]

                # Invalid patch : outside right epipolar image
                if np.sum(neg_patch_right != 0) != 0:
                    disp_map[row, col] = -9999
            # Negative example cannot be created
            else:
                disp_map[row, col] = -9999

    return disp_map


def save_dataset(img, sample, img_name, img_file, sample_file):
    """
    Save the sample in hdf5 files :
        - images are saved in the img_file file: creation of a dataset for each image pair
        - sample are saved in the sample_file file : creation of dataset containing valid pixels

    The dataset name is the ground truth file ( exemple : JAX_004_009_007_LEFT_DSP.tif )

    :param img: images
    :type img: np.array (2, 1024, 1024, 3) ( 2 = left image, right image)
    :param sample: samples of the image
    :type sample: np.array(number of valid pixels for all the images, 4).
        The last dimension is : number of the image, row, col, disparity for the pixel p(row, col)
    :param img_name: name of the current image pair ( name of the gt disparity )
    :type img_name: string
    :param img_file: image database file
    :type img_file: hdf5 file
    :param sample_file: training or testing database file
    :type sample_file: hdf5 file
    """
    sample_file.create_dataset(img_name, data=sample)
    img_file.create_dataset(img_name, data=img)


def fusion_contest(input_dir, output):
    """
    Preprocess and create data fusion contest hdf5 database

    :param input_dir: path to the  input directory
    :type input_dir: string
    :param output: output directory
    :type output: string
    """
    img_file = h5py.File(os.path.join(output, "images_training_dataset_fusion_contest.hdf5"), "w")
    training_file = h5py.File(os.path.join(output, "training_dataset_fusion_contest.hdf5"), "w")
    img_testing_file = h5py.File(os.path.join(output, "images_testing_dataset_fusion_contest.hdf5"), "w")
    testing_file = h5py.File(os.path.join(output, "testing_dataset_fusion_contest.hdf5"), "w")

    gt = glob.glob(input_dir + "/*/left_epipolar_disp.tif")

    nb_img = len(gt)
    # Shuffle the file list
    indices = np.arange(nb_img)
    np.random.seed(0)
    np.random.shuffle(indices)
    gt = [gt[i] for i in indices]

    # 90 % Training, 10 % Testing
    end_training = int(nb_img * 0.9)

    for num_image in range(nb_img):
        name_image = gt[num_image].split(input_dir)[1].split("/")[1]
        path_image = gt[num_image].split("left_epipolar_disp.tif")[0]

        # Read images
        left = rasterio.open(os.path.join(path_image, "left_epipolar_image.tif")).read(1)
        left_mask = rasterio.open(os.path.join(path_image, "left_epipolar_mask.tif")).read(1)
        right = rasterio.open(os.path.join(path_image, "right_epipolar_image.tif")).read(1)
        right_mask = rasterio.open(os.path.join(path_image, "right_epipolar_mask.tif")).read(1)
        dsp = rasterio.open(gt[num_image]).read(1)
        mask_dsp = rasterio.open(os.path.join(path_image, "left_epipolar_disp_mask.tif")).read(1)
        cross_checking = rasterio.open(os.path.join(path_image, "valid_disp.tif")).read(1)

        # Mask disparities
        mask_disp = compute_mask(dsp, left_mask, right_mask, 11)
        # Remove invalid pixels : invalidated by cross-checking mask and with invalid disparity
        mask_disp[np.where(cross_checking == 255)] = -9999
        mask_disp[np.where(mask_dsp == 255)] = -9999

        # Change the disparity convention to left(x,y) = right(x-d,y)
        mask_disp *= -1
        # Remove invalid disparity
        valid_row, valid_col = np.where(mask_disp != 9999)

        # Red band selection
        left = np.squeeze(left[0, :, :])
        right = np.squeeze(right[0, :, :])

        # Normalization
        valid_left = np.where(left_mask == 0)
        valid_right = np.where(right_mask == 0)
        left[valid_left] = (left[valid_left] - left[valid_left].mean()) / left[valid_left].std()
        right[valid_right] = (right[valid_right] - right[valid_right].mean()) / right[valid_right].std()

        # data np.array of shape ( number of valid pixels the current image, 4 )
        # 4 = number of the image, row, col, disparity for the pixel p(row, col)
        valid_disp = np.column_stack(
            (np.zeros_like(valid_row) + num_image, valid_row, valid_col, mask_disp[valid_row, valid_col])
        ).astype(np.float32)

        # img of shape (2, 2048, 2048, 3)
        img = np.stack((left, right), axis=0)
        if num_image > end_training:
            save_dataset(img, valid_disp, name_image, img_testing_file, testing_file)
        else:
            save_dataset(img, valid_disp, name_image, img_file, training_file)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Script for creating the training data fusion contest database. "
        "it will create the following files: "
        "- training_dataset_fusion_contest.hdf5, which contains training"
        " coordinates of the valid pixels and their disparity."
        "- testing_dataset_fusion_contest.hdf5, which contains testing "
        "coordinates of the valid pixels and their disparity."
        "- images_training_dataset_fusion_contest.hdf5, which contains the red"
        " band normalized training images"
        "- images_testing_dataset_fusion_contest.hdf5, which contains the red"
        " band normalized testing images"
    )
    parser.add_argument("input_data", help="Path to the input directory containing the data")
    parser.add_argument("output_dir", help="Path to the output directory ")
    args = parser.parse_args()

    fusion_contest(args.input_data, args.output_dir)



# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains the mc-cnn fast network
"""
# pylint:disable=too-few-public-methods

from torch import nn
import torch
import torch.nn.functional as F


class FastMcCnn(nn.Module):
    """
    Define the mc_cnn fast neural network

    """

    def __init__(self):
        super().__init__()
        self.in_channels = 1
        self.num_conv_feature_maps = 64
        self.conv_kernel_size = 3

        self.conv_blocks = nn.Sequential(
            nn.Conv2d(
                in_channels=self.in_channels, out_channels=self.num_conv_feature_maps, kernel_size=self.conv_kernel_size
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
        )

    # pylint: disable=arguments-differ
    # pylint: disable=no-else-return
    def forward(self, sample, training):
        """
        Forward function

        :param sample: sample
        :type sample:
            - if training mode :
                - normalized patch : torch ( batch_size, 3, 11, 11) with : 3 is the left patch, right positive patch,
                    right negative patch, 11 the patch
            - else :
                - normalized image torch(row, col)
        :param training: training mode
        :type training: bool
        :return:

            - if training mode : left, right positive and right negative features
            - else : extracted features
        :rtype:
            - if training mode : tuple([batch_size, 64, 1, 1], [batch_size, 64, 1, 1], [batch_size, 64, 1, 1])
            - else : torch(64, row, col)
        """
        if training:
            left = self.conv_blocks(sample[:, 0:1, :, :])
            # left of shape : torch.Size([batch_size, 64, 1, 1])
            left = F.normalize(left, p=2, dim=1)

            pos = self.conv_blocks(sample[:, 1:2, :, :])
            # pos of shape : torch.Size([batch_size, 64, 1, 1])
            pos = F.normalize(pos, p=2, dim=1)

            neg = self.conv_blocks(sample[:, 2:3, :, :])
            # neg of shape : torch.Size([batch_size, 64, 1, 1])
            neg = F.normalize(neg, p=2, dim=1)

            return left, pos, neg

        # Testing mode
        else:
            # Disabling gradient calculation in evaluation mode. It will reduce memory consumption
            with torch.no_grad():
                # Because input shape of nn.Conv2d is (Batch_size, Channel, H, W), we add 2 dimensions
                features = self.conv_blocks(sample.unsqueeze(0).unsqueeze(0))
                return torch.squeeze(F.normalize(features, p=2, dim=1))



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
This module contains all functions to train mc-cnn fast and accurate networks
"""

import argparse
import os
import errno
import json
import copy
import torch
from torch import nn, optim
from torch.utils import data


from mc_cnn.model.mc_cnn_accurate import AccMcCnn
from mc_cnn.model.mc_cnn_fast import FastMcCnn
from mc_cnn.dataset_generator.middlebury_generator import MiddleburyGenerator
from mc_cnn.dataset_generator.datas_fusion_contest_generator import DataFusionContestGenerator

device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")


def mkdir_p(path):
    """
    Create a directory without complaining if it already exists.
    """
    try:
        os.makedirs(path)
    except OSError as exc:  # requires Python > 2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def load_dataset(cfg):
    """
    Load training and testing data.

    :param cfg: configuration
    :type cfg: dict
    """

    # Testing configuration : deactivate data augmentation
    test_cfg = copy.deepcopy(cfg)
    test_cfg["transformation"] = False

    if cfg["dataset"] == "middlebury":
        training_loader = MiddleburyGenerator(cfg["training_sample"], cfg["training_image"], cfg)
        testing_loader = MiddleburyGenerator(cfg["testing_sample"], cfg["testing_image"], test_cfg)
    elif cfg["dataset"] == "data_fusion_contest":
        training_loader = DataFusionContestGenerator(cfg["training_sample"], cfg["training_image"], cfg)
        testing_loader = DataFusionContestGenerator(cfg["testing_sample"], cfg["testing_image"], test_cfg)
    else:
        raise ValueError(
            f"dataset key {cfg['dataset']} does not correspond to one of the options in the list "
            "['middlebury', 'data_fusion_contest'] ."
        )

    return training_loader, testing_loader


def train_mc_cnn_fast(cfg, output_dir, params):
    """
    Train the fast mc_cnn network

    :param cfg: configuration
    :type cfg: dict
    :param output_dir: output directory
    :type output_dir: string
    :param params: params for DataLoader
    :type params: dict
    """
    # Create the output directory
    mkdir_p(output_dir)
    save_cfg(output_dir, cfg)

    # Create the network
    net = FastMcCnn()
    net.to(device)

    criterion = nn.MarginRankingLoss(margin=0.2, reduction="mean")

    optimizer = optim.SGD(net.parameters(), lr=0.002, momentum=0.9)

    # lr = 0.002 if epoch < 9
    # lr = 0.0002 if 9 <= epoch < 18 ...
    scheduler = torch.optim.lr_scheduler.StepLR(optimizer, 9, gamma=0.1)

    # Load training and testing data
    training_loader, testing_loader = load_dataset(cfg)

    training_generator = data.DataLoader(training_loader, **params)
    testing_generator = data.DataLoader(testing_loader, **params)

    cos = nn.CosineSimilarity(dim=1, eps=1e-6)

    nb_epoch = 14
    training_loss = []
    testing_loss = []
    for epoch in range(nb_epoch):
        print("-------- Fast epoch" + str(epoch) + " ------------")

        train_epoch_loss = 0.0
        test_epoch_loss = 0.0
        net.train()

        for _, batch in enumerate(training_generator, 0):
            # zero the parameter gradients
            optimizer.zero_grad()

            left, pos, neg = net(batch.to(device=device, dtype=torch.float), training=True)

            # Cosine  similarity
            output_positive = cos(left, pos)
            output_negative = cos(left, neg)

            target = torch.ones(batch.size(0))
            loss = criterion.forward(output_positive, output_negative, target.to(device=device, dtype=torch.float))
            loss.backward()
            optimizer.step()

            train_epoch_loss += loss.item() * batch.size(0)

        training_loss.append(train_epoch_loss / len(training_loader))
        scheduler.step(epoch)

        net.eval()
        for _, batch in enumerate(testing_generator, 0):
            # zero the parameter gradients
            optimizer.zero_grad()

            left, pos, neg = net(batch.to(device=device, dtype=torch.float), training=True)

            # Cosine  similarity
            output_positive = cos(left, pos)
            output_negative = cos(left, neg)

            target = torch.ones(batch.size(0))
            loss = criterion.forward(output_positive, output_negative, target.to(device=device, dtype=torch.float))

            test_epoch_loss += loss.item() * batch.size(0)

        testing_loss.append(test_epoch_loss / len(testing_loader))

        # Save the network, optimizer, scheduler at each epoch
        torch.save(
            {
                "model": net.state_dict(),
                "optimizer": optimizer.state_dict(),
                "scheduler": scheduler.state_dict(),
                "epoch": epoch,
                "train_epoch_loss": train_epoch_loss / len(training_loader),
                "test_epoch_loss": test_epoch_loss / len(testing_loader),
            },
            os.path.join(output_dir, "mc_cnn_fast_epoch" + str(epoch) + ".pt"),
        )


def train_mc_cnn_acc(cfg, output_dir, params):
    """
    Train the accurate mc_cnn network

    :param cfg: configuration
    :type cfg: dict
    :param output_dir: output directory
    :type output_dir: string
    :param params: params for DataLoader
    :type params: dict
    """
    # Create the output directory
    mkdir_p(output_dir)
    save_cfg(output_dir, cfg)

    # Create the network
    net = AccMcCnn()
    net.to(device)

    criterion = nn.BCELoss(reduction="mean")

    optimizer = optim.SGD(net.parameters(), lr=0.003, momentum=0.9)

    # lr = 0.003 if epoch < 10
    # lr = 0.0003 if 10 <= epoch < 18 ...
    scheduler = torch.optim.lr_scheduler.StepLR(optimizer, 10, gamma=0.1)

    # Load training and testing data
    training_loader, testing_loader = load_dataset(cfg)

    training_generator = data.DataLoader(training_loader, **params)
    testing_generator = data.DataLoader(testing_loader, **params)

    nb_epoch = 14
    training_loss = []
    testing_loss = []
    for epoch in range(nb_epoch):
        print("-------- Accurate epoch" + str(epoch) + " ------------")

        train_epoch_loss = 0.0
        test_epoch_loss = 0.0
        net.train()

        for _, batch in enumerate(training_generator, 0):
            # zero the parameter gradients
            optimizer.zero_grad()

            pos, neg = net(batch.to(device=device, dtype=torch.float))

            sample = torch.cat((pos, neg), dim=0)
            sample = torch.squeeze(sample)

            target = torch.cat((torch.ones(batch.size(0)), torch.zeros(batch.size(0))), dim=0)

            loss = criterion.forward(sample, target.to(device=device, dtype=torch.float))
            loss.backward()
            optimizer.step()

            train_epoch_loss += loss.item() * batch.size(0)

        training_loss.append(train_epoch_loss / len(training_loader))
        scheduler.step(epoch)

        net.eval()
        for _, batch in enumerate(testing_generator, 0):
            # zero the parameter gradients
            optimizer.zero_grad()

            pos, neg = net(batch.to(device=device, dtype=torch.float))

            sample = torch.cat((pos, neg), dim=0)
            sample = torch.squeeze(sample)

            target = torch.cat((torch.ones(batch.size(0)), torch.zeros(batch.size(0))), dim=0)

            loss = criterion.forward(sample, target.to(device=device, dtype=torch.float))

            test_epoch_loss += loss.item() * batch.size(0)

        testing_loss.append(test_epoch_loss / len(testing_loader))

        # Save the network, optimizer, scheduler at each epoch
        torch.save(
            {
                "model": net.state_dict(),
                "optimizer": optimizer.state_dict(),
                "scheduler": scheduler.state_dict(),
                "epoch": epoch,
                "train_epoch_loss": train_epoch_loss / len(training_loader),
                "test_epoch_loss": test_epoch_loss / len(testing_loader),
            },
            os.path.join(output_dir, "mc_cnn_acc_epoch" + str(epoch) + ".pt"),
        )


def read_config_file(config_file):
    """
    Read a json configuration file

    :param config_file: path to a json file containing the algorithm parameters
    :type config_file: string
    :return: the configuration
    :rtype: dict
    """
    with open(config_file, "r", encoding="utf-8") as file:
        user_configuration = json.load(file)
    return user_configuration


def save_cfg(output, configuration):
    """
    Save user configuration in the json file : config.json

    :param output: output directory
    :param configuration: user configuration
    """
    with open(os.path.join(output, "config.json"), "w", encoding="utf-8") as file:
        json.dump(configuration, file, indent=2)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("injson", help="Input json file")
    parser.add_argument("outdir", help="Output directory")
    args = parser.parse_args()

    user_cfg = read_config_file(args.injson)

    # params for DataLoader
    batch_size = 128  # pylint: disable=C0103
    data_loader_params = {"batch_size": batch_size, "shuffle": True}

    if user_cfg["network"] == "fast":
        train_mc_cnn_fast(user_cfg, args.outdir, data_loader_params)
    else:
        train_mc_cnn_acc(user_cfg, args.outdir, data_loader_params)



# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains the mc-cnn accurate network
"""
# pylint:disable=too-few-public-methods

from torch import nn
import torch
import numpy as np


class AccMcCnn(nn.Module):
    """
    Define the mc_cnn accurate neural network for training

    """

    def __init__(self):
        super().__init__()
        self.in_channels = 1
        self.num_conv_feature_maps = 112
        self.conv_kernel_size = 3

        # Extract images features
        self.conv_blocks = nn.Sequential(
            nn.Conv2d(
                in_channels=self.in_channels, out_channels=self.num_conv_feature_maps, kernel_size=self.conv_kernel_size
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
        )

        # Compute similarity score
        self.fl_blocks = nn.Sequential(
            nn.Linear(in_features=224, out_features=384),
            nn.ReLU(),
            nn.Linear(in_features=384, out_features=384),
            nn.ReLU(),
            nn.Linear(in_features=384, out_features=384),
            nn.ReLU(),
            nn.Linear(in_features=384, out_features=1),
            nn.Sigmoid(),
        )

    # pylint: disable=arguments-differ
    def forward(self, sample):
        """
        Forward function

        :param sample: normalized patch
        :type sample: torch ( batch_size, 3, 11, 11) with : 3 is the left patch, right positive patch, right negative
            patch, 11 the patch size
        :return: similarity score for positive sample, similarity score for negative sample
        :rtype: tuple(torch.Tensor, torch.Tensor)
        """
        # Extract images features
        left = self.conv_blocks(sample[:, 0:1, :, :])
        # left of shape : torch.Size([batch_size, 112, 1, 1])

        pos = self.conv_blocks(sample[:, 1:2, :, :])
        # pos of shape : torch.Size([batch_size, 112, 1, 1])

        neg = self.conv_blocks(sample[:, 2:3, :, :])
        # neg of shape : torch.Size([batch_size, 112, 1, 1])

        # Compute similarity score
        # Positive output
        pos_sample = torch.cat((left, pos), dim=1)
        pos_sample = torch.squeeze(pos_sample)
        pos_sample = self.fl_blocks(pos_sample)

        # Negative output
        neg_sample = torch.cat((left, neg), dim=1)
        neg_sample = torch.squeeze(neg_sample)
        neg_sample = self.fl_blocks(neg_sample)

        return pos_sample, neg_sample


class AccMcCnnInfer(nn.Module):
    """
    Define the mc_cnn accurate neural network for inference

    """

    def __init__(self):
        super().__init__()
        self.in_channels = 1
        self.num_conv_feature_maps = 112
        self.conv_kernel_size = 3

        # Extract images features
        self.conv_blocks = nn.Sequential(
            nn.Conv2d(
                in_channels=self.in_channels, out_channels=self.num_conv_feature_maps, kernel_size=self.conv_kernel_size
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
            nn.Conv2d(
                in_channels=self.num_conv_feature_maps,
                out_channels=self.num_conv_feature_maps,
                kernel_size=self.conv_kernel_size,
            ),
            nn.ReLU(),
        )

        # Compute similarity score
        self.fl_blocks = nn.Sequential(
            nn.Linear(in_features=224, out_features=384),
            nn.ReLU(),
            nn.Linear(in_features=384, out_features=384),
            nn.ReLU(),
            nn.Linear(in_features=384, out_features=384),
            nn.ReLU(),
            nn.Linear(in_features=384, out_features=1),
            nn.Sigmoid(),
        )

    # pylint: disable=arguments-differ
    def forward(self, left, right, disp_min, disp_max):
        """
        Extract left and right features and computes the cost volume for a pair of images

        :param left: left image (normalized)
        :type left: torch (row, col)
        :param right: right image (normalized)
        :type right: torch (row, col)
        :param disp_min: minimal disparity
        :type disp_min: torch
        :param disp_max: maximal disparity
        :type disp_max: torch
        :return: return the cost volume ( similarity score is converted to a matching cost )
        :rtype: np.array 3D ( row, col, disp)
        """
        # Disabling gradient calculation in evaluation mode. It will reduce memory consumption
        with torch.no_grad():
            # Because input shape of nn.Conv2d is (Batch_size, Channel, H, W), we add 2 dimensions
            # Shape left_features and right_features is [1, 112, row-10, col-10]
            left_features = self.conv_blocks(left.unsqueeze(0).unsqueeze(0))
            right_features = self.conv_blocks(right.unsqueeze(0).unsqueeze(0))

            cv = self.computes_cost_volume_mc_cnn_accurate(
                left_features, right_features, disp_min, disp_max, self.compute_cost_mc_cnn_accurate
            )

            return cv

    @staticmethod
    def computes_cost_volume_mc_cnn_accurate(left_features, right_features, disp_min, disp_max, measure):
        """
        Computes the cost volume using the left and right features computing by mc_cnn accurate

        :param left_features: left features
        :type left_features: Tensor of shape (1, 112, row, col)
        :param right_features: right features
        :type right_features: Tensor of shape (1, 112, 64, row, col)
        :param measure: measure to apply
        :type measure: function
        :return: the cost volume ( similarity score is converted to a matching cost )
        :rtype: 3D np.array (row, col, disp)
        """
        disparity_range = torch.arange(disp_min, disp_max + 1)

        # Allocate the numpy cost volume cv = (disp, col, row), for efficient memory management
        cv = np.zeros((len(disparity_range), left_features.shape[3], left_features.shape[2]), dtype=np.float32)
        cv += np.nan

        _, _, _, nx_left = left_features.shape
        _, _, _, nx_right = right_features.shape

        # Disabling gradient calculation in evaluation mode. It will reduce memory consumption
        with torch.no_grad():
            for disp in disparity_range:
                # range in the left image
                left = (max(0 - disp, 0), min(nx_left - disp, nx_left))
                # range in the right image
                right = (max(0 + disp, 0), min(nx_right + disp, nx_right))
                index_d = int(disp - disp_min)

                cv[index_d, left[0] : left[1], :] = np.swapaxes(
                    measure(left_features[:, :, :, left[0] : left[1]], right_features[:, :, :, right[0] : right[1]]),
                    0,
                    1,
                )

        # The minus sign converts the similarity score to a matching cost
        cv *= -1

        return np.swapaxes(cv, 0, 2)

    def compute_cost_mc_cnn_accurate(self, left_features, right_features):
        """
        Compute the cost between the left and right features using the last part of the mc_cnn accurate

        :param left_features: left features
        :type left_features: Tensor of shape (1, 112, row, col)
        :param right_features: right features
        :type right_features: Tensor of shape (1, 112, row, col)
        :return: the cost
        :rtype:  Tensor of shape (col, row)
        """
        sample = torch.cat((left_features, right_features), dim=1)
        # Tanspose because input of nn.Linear is(batch_size, *, in_features)
        sample = self.fl_blocks(sample.permute(0, 2, 3, 1))
        sample = torch.squeeze(sample)
        return sample.cpu().detach().numpy()



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2015, Jure Zbontar <jure.zbontar@gmail.com>
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains all functions to generate the training and testing dataset on Middlebury
"""

import argparse
import os
import logging
import re
import numpy as np
from numba import njit, prange
import h5py
import rasterio


def load_pfm(fname):
    """
    Load a PFM file into a Numpy array.

    :param fname: path to the PFM file
    :type fname: string
    :return: data of the PFM file
    :rtype: tuple(np.array (row, col) , scale factor )
    """
    color = None
    width = None
    height = None
    scale = None
    endian = None

    with open(fname, "rb") as file:
        header = file.readline().rstrip().decode("latin-1")
        if header == "PF":
            color = True
        elif header == "Pf":
            color = False
        else:
            raise OSError("Not a PFM file.")

        dim_match = re.match(r"^(\d+)\s(\d+)\s$", file.readline().decode("latin-1"))
        if dim_match:
            width, height = map(int, dim_match.groups())
        else:
            raise OSError("Malformed PFM header.")

        scale = float(file.readline().rstrip().decode("latin-1"))
        if scale < 0:  # little-endian
            endian = "<"
            scale = -1 * scale
        else:
            endian = ">"  # big-endian

        data = np.fromfile(file, endian + "f")
        shape = (height, width, 3) if color else (height, width)
    return np.flipud(np.reshape(data, shape)), scale


def read_im(fname, downsample):
    """
    Read image, apply gray conversion, normalize image

    :param fname: path to the image
    :type fname: string
    :param downsample: downsample the image
    :type downsample: bool
    :return: data of the file
    :rtype: np.array (1, row, col)
    """
    image = rasterio.open(fname).read()
    if downsample:
        image = image[:, ::2, ::2]
    # Gray conversion : [Stereo Matching by Training a Convolutional NeuralNetwork to Compare Image Patches]
    # Our initial experiments suggested that using color information does not improve the quality of the disparity maps
    # therefore, we converted all color images to grayscale.

    # The two latest images are shaped (1, n_lig, n_col)
    #  if the first dimension do not fit, exchange the dimensions
    #  otherwise, remove the first dimension
    if (len(image.shape) == 3) and (image.shape[0] != 1):
        image = image.transpose(1, 2, 0)
        image = np.dot(image[:, :, :], [0.299, 0.587, 0.114])
    else:
        image = image[0, :, :]
    # Normalize
    image = (image - image.mean()) / image.std()

    return image[None]


@njit(parallel=True)
def compute_mask(left_disp, left_row_disp, right_disp, patch_size):
    """
    Apply cross-checking, and invalidate pixels with incomplete patch

    :param left_disp: Left disparity
    :type left_disp: numpy.array (row, col)
    :param left_row_disp: Left column disparity
    :type left_row_disp: numpy.array (row, col)
    :param right_disp: Right disparity
    :type right_disp: numpy.array (row, col)
    :param patch_size: patch size
    :type patch_size: int
    :return: Result of the cross-checking with the convention : invalid pixels = 0, valid pixels = 1
    :rtype: numpy.array (row, col)
    """
    row, col = left_disp.shape
    mask = np.zeros((row, col), dtype=np.float32)
    rad = int(patch_size / 2)
    # 6 = maximum negative displacement for creating negative exemple during training
    rad += 6

    for row_idx in prange(rad, row - rad):  # pylint: disable=not-an-iterable
        for col_idx in prange(rad, col - rad):  # pylint: disable=not-an-iterable
            disp_left_x = left_disp[row_idx, col_idx]

            disp_left_y = 0
            if left_row_disp is not None:
                disp_left_y = left_row_disp[row_idx, col_idx]

            if disp_left_x != np.inf:
                idx_right_x = int(col_idx + (-1 * disp_left_x))
                idx_right_y = int(row_idx + disp_left_y)

                if rad < idx_right_x < (col - rad) and rad < idx_right_y < (row - rad):
                    disp_right_x = right_disp[idx_right_y, idx_right_x]
                    if abs(disp_left_x - disp_right_x) < 1:
                        mask[row_idx, col_idx] = 1
    return mask


def save_dataset(img, sample, num_img, img_file, sample_file):
    """
    Save the dataset in hdf5 files :
        - images are saved in the img_file file: creation of a group of name num_img that contains number of exposures
            dataset
        - sample are saved in the sample_file file : creation of dataset name num_img

    :param img: images
    :type img: list( np.array(number of exposures, 2, row, col)). Size of the list is the number of illuminations of
        the 2 views
    :param sample: samples of the image
    :type sample: np.array(number of valid pixels for all the images, 4).
        The last dimension is : the image index (num_img), row, col, disparity for the pixel p(row, col)
    :param num_img: image number
    :type num_img: int
    :param img_file: image database file
    :type img_file: hdf5 file
    :param sample_file: training or testing database file
    :type sample_file: hdf5 file
    """
    grp = img_file.create_group(str(num_img))
    for light, __ in enumerate(img):
        __ = grp.create_dataset(str(light), data=img[light])  # pylint: disable=unnecessary-list-index-lookup

    sample_file.create_dataset(str(num_img), data=sample)


# pylint: disable=too-many-locals, too-many-branches, too-many-statements, too-many-function-args
def middleburry(in_dir_2014, in_dir_2006, in_dir_2005, in_dir_2003, in_dir_2001, output_dir):
    """
    Preprocess and create middlebury hdf5 database

    :param in_dir_2014: path to the middlebury 2014 dataset
    :type in_dir_2014: string
    :param in_dir_2006: path to the middlebury 2006 dataset
    :type in_dir_2006: string
    :param in_dir_2005: path to the middlebury 2005 dataset
    :type in_dir_2005: string
    :param in_dir_2003: path to the middlebury 2003 dataset
    :type in_dir_2003: string
    :param in_dir_2001: path to the middlebury 2001 dataset
    :type in_dir_2001: string
    :param output_dir: output directory
    :type output_dir: string
    """
    patch_size = 11
    # Creating hdf5 file
    img_file = h5py.File(os.path.join(output_dir, "images.hdf5"), "w")
    training_file = h5py.File(os.path.join(output_dir, "training_dataset.hdf5"), "w")
    testing_file = h5py.File(os.path.join(output_dir, "testing_dataset.hdf5"), "w")

    logging.info("Middlebury 2014")
    # --------------- Middlebury 2014 dataset ---------------

    # Testing dataset = 'Adirondack-imperfect', 'Backpack-imperfect', 'Bicycle1-imperfect', 'Cable-imperfect',
    # 'Classroom1-imperfect', 'Couch-imperfect', 'Flowers-imperfect'
    test_ds_range = np.arange(0, 7)

    # Training dataset = 'Jadeplant-imperfect', 'Mask-imperfect', 'Motorcycle-imperfect',
    # 'Piano-imperfect', 'Pipes-imperfect', 'Playroom-imperfect', 'Playtable-imperfect', 'Recycle-imperfect',
    # 'Shelves-imperfect', 'Shopvac-imperfect', 'Sticks-imperfect', 'Storage-imperfect', 'Sword1-imperfect',
    # 'Sword2-imperfect', 'Umbrella-imperfect', 'Vintage-imperfect'

    num_image = 0

    for directory in sorted(os.listdir(in_dir_2014)):
        base1 = os.path.join(in_dir_2014, directory)

        left = read_im(os.path.join(base1, "im0.png"), True)
        right = read_im(os.path.join(base1, "im1.png"), True)
        im_tensor = [np.expand_dims(np.concatenate((left, right)), axis=0)]

        right_exp = read_im(os.path.join(base1, "im1E.png"), True)
        im_tensor.append(np.expand_dims(np.concatenate((left, right_exp)), axis=0))

        right_lum = read_im(os.path.join(base1, "im1L.png"), True)
        im_tensor.append(np.expand_dims(np.concatenate((left, right_lum)), axis=0))

        base2 = os.path.join(base1, "ambient")
        num_light = len(os.listdir(base2))

        for light in range(num_light):
            imgs = []

            base4 = os.path.join(base2, "L" + str(light + 1))
            exp = os.listdir(base4)
            num_exp = int(len(exp) / 2)

            for elem in range(0, num_exp):
                left = read_im(base4 + "/im0e" + str(elem) + ".png", True)
                right = read_im(base4 + "/im1e" + str(elem) + ".png", True)
                imgs.append(np.concatenate((left, right)))

            im_tensor.append(np.concatenate(imgs).reshape(num_exp, 2, left.shape[1], left.shape[2]))

        # Read ground truth disparity
        left_disp, __ = load_pfm(os.path.join(base1, "disp0.pfm"))
        # Downsample
        left_disp = left_disp[::2, ::2]
        right_disp, __ = load_pfm(os.path.join(base1, "disp1.pfm"))
        # Downsample
        right_disp = right_disp[::2, ::2]

        # Left GT y-disparities
        left_row_disp, __ = load_pfm(os.path.join(base1, "disp0y.pfm"))
        # Downsample
        left_row_disp = left_row_disp[::2, ::2]

        # Remove occluded pixels
        mask = compute_mask(left_disp, left_row_disp, right_disp, patch_size)
        left_disp[mask != 1] = 0

        non_zero_y_idx, non_zero_x_idx = np.nonzero(mask)

        # data np.array of shape ( number of valid pixels for all the images, 4 )
        # 4 = the image index, row, col, disparity for the pixel p(row, col)
        data = np.column_stack(
            (
                np.zeros_like(non_zero_y_idx) + num_image,
                non_zero_y_idx,
                non_zero_x_idx,
                left_disp[non_zero_y_idx, non_zero_x_idx],
            )
        ).astype(np.float32)

        if num_image in test_ds_range:
            save_dataset(im_tensor, data, num_image, img_file, testing_file)
        else:
            save_dataset(im_tensor, data, num_image, img_file, training_file)
        num_image += 1

    logging.info("Middlebury 2006")
    # --------------- Middlebury 2006 dataset ---------------
    for directory in sorted(os.listdir(in_dir_2006)):
        im_tensor = []

        base1 = os.path.join(in_dir_2006, directory)

        for light in range(3):
            imgs = []
            for exp in (0, 1, 2):
                base3 = os.path.join(base1, "Illum" + str(light + 1), "Exp" + str(exp))
                left = read_im(os.path.join(base3, "view1.png"), False)
                right = read_im(os.path.join(base3, "view5.png"), False)
                imgs.append(left)
                imgs.append(right)

            _, height, width = imgs[0].shape
            # im_tensor is a list of size = 1 + number of light, im_tensor[0].shape = (3, 2, row, col )
            im_tensor.append(np.concatenate(imgs).reshape(len(imgs) // 2, 2, height, width))

        left_disp = rasterio.open(base1 + "/disp1.png").read().astype(np.float32)
        right_disp = rasterio.open(base1 + "/disp5.png").read().astype(np.float32)

        # Takes the first dimension (shaped (1, n_l, n_c))
        # In the half-size versions, the intensity values of the disparity maps need to be divided by 2
        left_disp = left_disp[0, :, :] / 2
        right_disp = right_disp[0, :, :] / 2

        left_disp[left_disp == 0] = np.inf
        right_disp[right_disp == 0] = np.inf

        mask = compute_mask(left_disp, None, right_disp, patch_size)
        left_disp[mask != 1] = 0

        non_zero_y_idx, non_zero_x_idx = np.nonzero(mask)

        data = np.column_stack(
            (
                np.zeros_like(non_zero_y_idx) + num_image,
                non_zero_y_idx,
                non_zero_x_idx,
                left_disp[non_zero_y_idx, non_zero_x_idx],
            )
        ).astype(np.float32)
        save_dataset(im_tensor, data, num_image, img_file, training_file)
        num_image += 1

    logging.info("Middlebury 2005")
    # --------------- Middlebury 2005 dataset ---------------
    for directory in sorted(os.listdir(in_dir_2005)):
        im_tensor = []

        base1 = os.path.join(in_dir_2005, directory)
        if not os.path.isfile(base1 + "/disp1.png"):
            continue

        for light in range(3):
            imgs = []
            for exp in (0, 1, 2):
                base3 = os.path.join(base1, "Illum" + str(light + 1), "Exp" + str(exp))
                left = read_im(os.path.join(base3, "view1.png"), False)
                right = read_im(os.path.join(base3, "view5.png"), False)
                imgs.append(left)
                imgs.append(right)

            _, height, width = imgs[0].shape
            # im_tensor is a list of size = 1 + number of light
            # im_tensor[0].shape = (3, 2, row, col )
            im_tensor.append(np.concatenate(imgs).reshape(len(imgs) // 2, 2, height, width))

        left_disp = rasterio.open(base1 + "/disp1.png").read().astype(np.float32)
        right_disp = rasterio.open(base1 + "/disp5.png").read().astype(np.float32)

        # Takes the first dimension (shaped (1, n_l, n_c));
        # In the half-size versions, the intensity values of the disparity maps need to be divided by 2
        left_disp = left_disp[0, :, :] / 2
        right_disp = right_disp[0, :, :] / 2

        left_disp[left_disp == 0] = np.inf
        right_disp[right_disp == 0] = np.inf

        mask = compute_mask(left_disp, None, right_disp, patch_size)
        left_disp[mask != 1] = 0

        non_zero_y_idx, non_zero_x_idx = np.nonzero(mask)

        data = np.column_stack(
            (
                np.zeros_like(non_zero_y_idx) + num_image,
                non_zero_y_idx,
                non_zero_x_idx,
                left_disp[non_zero_y_idx, non_zero_x_idx],
            )
        ).astype(np.float32)
        save_dataset(im_tensor, data, num_image, img_file, training_file)
        num_image += 1

    logging.info("Middlebury 2003")
    # --------------- Middlebury 2003 dataset ---------------
    for directory in ("conesH", "teddyH"):
        base1 = os.path.join(in_dir_2003, directory)

        im_tensor = []
        imgs = []

        left = read_im(base1 + "/im2.ppm", False)
        right = read_im(base1 + "/im6.ppm", False)
        _, height, width = left.shape

        imgs.append(left)
        imgs.append(right)
        im_tensor.append(np.concatenate(imgs).reshape(len(imgs) // 2, 2, height, width))

        left_disp = rasterio.open(base1 + "/disp2.pgm").read().astype(np.float32)
        right_disp = rasterio.open(base1 + "/disp6.pgm").read().astype(np.float32)

        # Takes the first dimension (shaped (1, n_l, n_c))
        # In the half-size versions, the intensity values of the disparity maps need to be divided by 2
        left_disp = left_disp[0, :, :] / 2
        right_disp = right_disp[0, :, :] / 2

        # Disparity 0 is invalid
        left_disp[left_disp == 0] = np.inf
        right_disp[right_disp == 0] = np.inf

        mask = compute_mask(left_disp, None, right_disp, patch_size)
        left_disp[mask != 1] = 0

        non_zero_y_idx, non_zero_x_idx = np.nonzero(mask)

        data = np.column_stack(
            (
                np.zeros_like(non_zero_y_idx) + num_image,
                non_zero_y_idx,
                non_zero_x_idx,
                left_disp[non_zero_y_idx, non_zero_x_idx],
            )
        ).astype(np.float32)
        save_dataset(im_tensor, data, num_image, img_file, training_file)
        num_image += 1

    logging.info("Middlebury 2001")
    # --------------- Middlebury 2001 dataset ---------------
    for directory in sorted(os.listdir(in_dir_2001)):
        if directory == "tsukuba":
            continue
        if directory == "map":
            fname_disp0, fname_disp1, fname_x0, fname_x1 = "disp0.pgm", "disp1.pgm", "im0.pgm", "im1.pgm"
        else:
            fname_disp0, fname_disp1, fname_x0, fname_x1 = "disp2.pgm", "disp6.pgm", "im2.ppm", "im6.ppm"

        base2 = os.path.join(in_dir_2001, directory)
        if os.path.isfile(os.path.join(base2, fname_disp0)):
            im_tensor = []
            imgs = []

            left = read_im(os.path.join(base2, fname_x0), False)
            right = read_im(os.path.join(base2, fname_x1), False)
            _, height, width = left.shape

            imgs.append(left)
            imgs.append(right)
            im_tensor.append(np.concatenate(imgs).reshape(len(imgs) // 2, 2, height, width))

            left_disp = rasterio.open(os.path.join(base2, fname_disp0)).read().astype(np.float32) / 8.0
            right_disp = rasterio.open(os.path.join(base2, fname_disp1)).read().astype(np.float32) / 8.0

            # Takes the first dimension (shaped (1, n_l, n_c))
            left_disp = left_disp[0, :, :]
            right_disp = right_disp[0, :, :]

            mask = compute_mask(left_disp, None, right_disp, patch_size)

            left_disp[mask != 1] = 0

            non_zero_y_idx, non_zero_x_idx = np.nonzero(mask)

            data = np.column_stack(
                (
                    np.zeros_like(non_zero_y_idx) + num_image,
                    non_zero_y_idx,
                    non_zero_x_idx,
                    left_disp[non_zero_y_idx, non_zero_x_idx],
                )
            ).astype(np.float32)
            save_dataset(im_tensor, data, num_image, img_file, training_file)
            num_image += 1


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Script for creating the middlebury database, creation of files : "
        "- training_dataset.hdf5, testing_dataset.hdf5 which contains the "
        "coordinates of the valid pixels and their disparity "
        "- images.hdf5 which contains the grayscale and normalized images "
    )
    parser.add_argument("input_dir_2014", help="Path to the input directory containing the 2014 Middlebury dataset")
    parser.add_argument("input_dir_2006", help="Path to the input directory containing the 2006 Middlebury dataset")
    parser.add_argument("input_dir_2005", help="Path to the input directory containing the 2005 Middlebury dataset")
    parser.add_argument("input_dir_2003", help="Path to the input directory containing the 2003 Middlebury dataset")
    parser.add_argument("input_dir_2001", help="Path to the input directory containing the 2001 Middlebury dataset")
    parser.add_argument("output_dir", help="Path to the output directory ")
    args = parser.parse_args()

    middleburry(
        args.input_dir_2014,
        args.input_dir_2006,
        args.input_dir_2005,
        args.input_dir_2003,
        args.input_dir_2001,
        args.output_dir,
    )



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
Packaging setup.py for compatibility
All packaging in setup.cfg, except setuptools_scm version activation
"""

from setuptools import setup

try:
    setup()
except Exception:
    print(
        "\n\nAn error occurred while building the project, "
        "please ensure you have the most updated version of setuptools, "
        "setuptools_scm and wheel with:\n"
        "   pip install -U setuptools setuptools_scm wheel\n\n"
    )
    raise



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains functions to test mc-cnn fast and accurate
"""


import numpy as np
import torch
from torch import nn

from mc_cnn.model.mc_cnn_fast import FastMcCnn
from mc_cnn.model.mc_cnn_accurate import AccMcCnnInfer


def point_interval(left_features, right_features, disp):
    """
    Computes the range of points over which the similarity measure will be applied

    :param left_features: left features
    :type left_features: Tensor of shape (64, row, col)
    :param right_features: right features
    :type right_features: Tensor of shape (64, row, col)
    :param disp: current disparity
    :type disp: float
    :return: the range of the left and right image over which the similarity measure will be applied
    :rtype: tuple
    """
    _, _, nx_left = left_features.shape
    _, _, nx_right = right_features.shape

    # range in the left image
    left = (max(0 - disp, 0), min(nx_left - disp, nx_left))
    # range in the right image
    right = (max(0 + disp, 0), min(nx_right + disp, nx_right))

    return left, right


def run_mc_cnn_fast(img_left, img_right, disp_min, disp_max, model_path):
    """
    Computes the cost volume for a pair of images with mc-cnn fast

    :param img_left: left image
    :type img_left: np.ndarray
    :param img_right: right image
    :type img_right: np.ndarray
    :param disp_min: minimum disparity
    :type disp_min: int
    :param disp_max: maximum disparity
    :type disp_max: int
    :param model_path: path to the trained network
    :type model_path: string
    :return: the cost volume ( similarity score is converted to a matching cost )
    :rtype: 3D np.array (row, col, disp)
    """
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    # Create the network
    net = FastMcCnn()
    # Load the network
    net.load_state_dict(torch.load(model_path, map_location=device)["model"])
    net.to(device)
    net.eval()

    # Normalize images
    left = (img_left - img_left.mean()) / img_left.std()

    right = (img_right - img_right.mean()) / img_right.std()

    # Extracts the image features by propagating the images in the mc_cnn fast network
    # Right and left features of shape : (64, row-10, col-10)
    left_features = net(torch.from_numpy(left).to(device=device, dtype=torch.float), training=False)
    right_features = net(torch.from_numpy(right).to(device=device, dtype=torch.float), training=False)

    cv = computes_cost_volume_mc_cnn_fast(left_features, right_features, disp_min, disp_max)

    return cv


def computes_cost_volume_mc_cnn_fast(left_features, right_features, disp_min, disp_max):
    """
    Computes the cost volume using the left and right features computing by mc_cnn fast

    :param left_features: left features
    :type left_features: Tensor of shape (64, row, col)
    :param right_features: right features
    :type right_features: Tensor of shape (64, row, col)
    :return: the cost volume ( similarity score is converted to a matching cost )
    :rtype: 3D np.array (row, col, disp)
    """
    # Construct the cost volume
    disparity_range = np.arange(disp_min, disp_max + 1)

    # Allocate the numpy cost volume cv = (disp, col, row), for efficient memory management
    cv = np.zeros((len(disparity_range), left_features.shape[2], left_features.shape[1]), dtype=np.float32)
    cv += np.nan

    cos = nn.CosineSimilarity(dim=0, eps=1e-6)

    for disp in disparity_range:
        # Columns range in left and right image
        left, right = point_interval(left_features, right_features, disp)
        ind_d = int(disp - disp_min)
        cv[ind_d, left[0] : left[1], :] = np.swapaxes(
            (
                cos(left_features[:, :, left[0] : left[1]], right_features[:, :, right[0] : right[1]])
                .cpu()
                .detach()
                .numpy()
            ),
            0,
            1,
        )

    # Releases cache memory
    torch.cuda.empty_cache()

    # The minus sign converts the similarity score to a matching cost
    cv *= -1

    return np.swapaxes(cv, 0, 2)


def run_mc_cnn_accurate(img_left, img_right, disp_min, disp_max, model_path):
    """
    Computes the cost volume for a pair of images with mc-cnn accurate

    :param img_left: left image
    :type img_left: np.ndarray
    :param img_right: right image
    :type img_right: np.ndarray
    :param disp_min: minimum disparity
    :type disp_min: int
    :param disp_max: maximum disparity
    :type disp_max: int
    :param model_path: path to the trained network
    :type model_path: string
    :return: the cost volume ( similarity score is converted to a matching cost )
    :rtype: 3D np.array (row, col, disp)
    """
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    # Create the network
    net = AccMcCnnInfer()
    # Load the network
    net.load_state_dict(torch.load(model_path, map_location=device)["model"])
    net.to(device)
    net.eval()

    # Normalize images
    left = (img_left - img_left.mean()) / img_left.std()

    right = (img_right - img_right.mean()) / img_right.std()

    cv = net(
        torch.from_numpy(left).to(device=device, dtype=torch.float),
        torch.from_numpy(right).to(device=device, dtype=torch.float),
        disp_min,
        disp_max,
    )
    # Releases cache memory
    torch.cuda.empty_cache()
    return cv



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains all functions to access MC-CNN weights
"""

try:
    from importlib.resources import files
except ImportError:
    from importlib_resources import files  # python<=3.8


AVAILABLE_WEIGHTS = {
    "fast": {"middlebury": "mc_cnn_fast_mb_weights.pt", "dfc": "mc_cnn_fast_data_fusion_contest.pt"},
    "accurate": {"middlebury": "mc_cnn_accurate_mb_weights.pt", "dfc": "mc_cnn_accurate_data_fusion_contest.pt"},
}


def get_weights(arch="fast", training_dataset="middlebury"):
    """
    Return the absolute path of MC-CNN weights according to network and training parameters

    :param arch: architecture of MC-CNN : "fast" or "accurate"
    :type arch: str
    :param training_dataset: training dataset of MC-CNN : "middlebury" of "dfc" (Data Fusion Contest)
    :type training_dataset: str
    :return: absolute path of MC-CNN weights (.pt file)
    :rtype: PosixPath
    """
    filename = AVAILABLE_WEIGHTS[arch][training_dataset]
    return files("mc_cnn.weights").joinpath(filename)



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2024 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA_MCCNN
#
#     https://github.com/CNES/Pandora_MCCNN
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains functions to test the cost volume create by mc_cnn
"""

# Needs refactoring to pytest completely and remove all pylint disable


import unittest
import numpy as np
import torch
from torch import nn

from mc_cnn.run import computes_cost_volume_mc_cnn_fast
from mc_cnn.model.mc_cnn_accurate import AccMcCnnInfer
from mc_cnn.model.mc_cnn_fast import FastMcCnn
from mc_cnn.dataset_generator.middlebury_generator import MiddleburyGenerator
from mc_cnn.dataset_generator.datas_fusion_contest_generator import DataFusionContestGenerator
from mc_cnn.weights import get_weights


# load-plugins=pylint.extensions.no_self_use
class TestMCCNN(unittest.TestCase):
    """
    TestMCCNN class allows to test the cost volume create by mc_cnn
    """

    def setUp(self):
        """
        Method called to prepare the test fixture

        """
        self.left_img_0 = np.tile(np.arange(13, dtype=np.float32), (13, 1))
        self.right_img_0 = np.tile(np.arange(13, dtype=np.float32), (13, 1)) + 1

        self.left_img_1 = np.tile(np.arange(13, dtype=np.float32), (13, 1))
        self.right_img_2 = np.tile(np.arange(13, dtype=np.float32), (13, 1)) - 1

    def test_computes_cost_volume_mc_cnn_fast(self):
        """ "
        Test the computes_cost_volume_mc_cnn_fast function

        """
        # create left and left features
        left_feature = torch.randn((64, 4, 4), dtype=torch.float64)
        right_features = torch.randn((64, 4, 4), dtype=torch.float64)

        cos = nn.CosineSimilarity(dim=0, eps=1e-6)

        # Create the ground truth cost volume (row, col, disp)
        cv_gt = np.full((4, 4, 5), np.nan)

        # disparity -2
        cv_gt[:, 2:, 0] = cos(left_feature[:, :, 2:], right_features[:, :, 0:2]).cpu().detach().numpy()
        # disparity -1
        cv_gt[:, 1:, 1] = cos(left_feature[:, :, 1:], right_features[:, :, 0:3]).cpu().detach().numpy()
        # disparity 0
        cv_gt[:, :, 2] = cos(left_feature[:, :, :], right_features[:, :, :]).cpu().detach().numpy()
        # disparity 1
        cv_gt[:, :3, 3] = cos(left_feature[:, :, :3], right_features[:, :, 1:4]).cpu().detach().numpy()
        # disparity 2
        cv_gt[:, :2, 4] = cos(left_feature[:, :, :2], right_features[:, :, 2:4]).cpu().detach().numpy()

        # The minus sign converts the similarity score to a matching cost
        cv_gt *= -1

        cv = computes_cost_volume_mc_cnn_fast(left_feature, right_features, -2, 2)

        # Check if the calculated cost volume is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_allclose(cv, cv_gt, rtol=1e-05)

    def test_computes_cost_volume_mc_cnn_fast_negative_disp(self):
        """ "
        Test the computes_cost_volume_mc_cnn_fast function with negative disparities

        """
        # create left and right features
        left_feature = torch.randn((64, 4, 4), dtype=torch.float64)
        right_features = torch.randn((64, 4, 4), dtype=torch.float64)

        cos = nn.CosineSimilarity(dim=0, eps=1e-6)

        # Create the ground truth cost volume (row, col, disp)
        cv_gt = np.full((4, 4, 4), np.nan)

        # disparity -4
        # all nan
        # disparity -3
        cv_gt[:, 3:, 1] = cos(left_feature[:, :, 3:], right_features[:, :, 0:1]).cpu().detach().numpy()
        # disparity -2
        cv_gt[:, 2:, 2] = cos(left_feature[:, :, 2:], right_features[:, :, 0:2]).cpu().detach().numpy()
        # disparity -1
        cv_gt[:, 1:, 3] = cos(left_feature[:, :, 1:], right_features[:, :, 0:3]).cpu().detach().numpy()

        # The minus sign converts the similarity score to a matching cost
        cv_gt *= -1

        cv = computes_cost_volume_mc_cnn_fast(left_feature, right_features, -4, -1)

        # Check if the calculated cost volume is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_allclose(cv, cv_gt, rtol=1e-05)

    def test_computes_cost_volume_mc_cnn_fast_positive_disp(self):
        """ "
        Test the computes_cost_volume_mc_cnn_fast function with positive disparities

        """
        # create left and right features
        left_feature = torch.randn((64, 4, 4), dtype=torch.float64)
        right_features = torch.randn((64, 4, 4), dtype=torch.float64)

        cos = nn.CosineSimilarity(dim=0, eps=1e-6)

        # Create the ground truth cost volume (row, col, disp)
        cv_gt = np.full((4, 4, 4), np.nan)

        # disparity 1
        cv_gt[:, :3, 0] = cos(left_feature[:, :, :3], right_features[:, :, 1:4]).cpu().detach().numpy()
        # disparity 2
        cv_gt[:, :2, 1] = cos(left_feature[:, :, :2], right_features[:, :, 2:4]).cpu().detach().numpy()
        # disparity 3
        cv_gt[:, :1, 2] = cos(left_feature[:, :, :1], right_features[:, :, 3:]).cpu().detach().numpy()
        # disparity 4
        # all nan

        # The minus sign converts the similarity score to a matching cost
        cv_gt *= -1

        cv = computes_cost_volume_mc_cnn_fast(left_feature, right_features, 1, 4)

        # Check if the calculated cost volume is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_allclose(cv, cv_gt, rtol=1e-05)

    def sad_cost(self, left_features, right_features):
        """
        Useful to test the computes_cost_volume_mc_cnn_accurate function
        """
        return torch.sum(abs(left_features[0, :, :, :] - right_features[0, :, :, :]), dim=0)

    def test_computes_cost_volume_mc_cnn_accurate(self):
        """ "
        Test the computes_cost_volume_mc_cnn_accurate function

        """
        # create left and right features
        left_feature = torch.randn((1, 112, 4, 4), dtype=torch.float64)
        right_features = torch.randn((1, 112, 4, 4), dtype=torch.float64)

        # Create the ground truth cost volume (row, col, disp)
        cv_gt = np.full((4, 4, 5), np.nan)

        # disparity -2
        cv_gt[:, 2:, 0] = self.sad_cost(left_feature[:, :, :, 2:], right_features[:, :, :, 0:2]).cpu().detach().numpy()
        # disparity -1
        cv_gt[:, 1:, 1] = self.sad_cost(left_feature[:, :, :, 1:], right_features[:, :, :, 0:3]).cpu().detach().numpy()
        # disparity 0
        cv_gt[:, :, 2] = self.sad_cost(left_feature[:, :, :, :], right_features[:, :, :, :]).cpu().detach().numpy()
        # disparity 1
        cv_gt[:, :3, 3] = self.sad_cost(left_feature[:, :, :, :3], right_features[:, :, :, 1:4]).cpu().detach().numpy()
        # disparity 2
        cv_gt[:, :2, 4] = self.sad_cost(left_feature[:, :, :, :2], right_features[:, :, :, 2:4]).cpu().detach().numpy()

        # The minus sign converts the similarity score to a matching cost
        cv_gt *= -1

        acc = AccMcCnnInfer()
        # Because input shape of nn.Conv2d is (Batch_size, Channel, H, W), we add 1 dimensions
        cv = acc.computes_cost_volume_mc_cnn_accurate(left_feature, right_features, -2, 2, self.sad_cost)

        # Check if the calculated cost volume is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_allclose(cv, cv_gt, rtol=1e-05)

    def test_computes_cost_volume_mc_cnn_accuratenegative_disp(self):
        """ "
        Test the computes_cost_volume_mc_cnn_accurate function with negative disparities

        """
        # create left and right features
        left_feature = torch.randn((1, 112, 4, 4), dtype=torch.float64)
        right_features = torch.randn((1, 112, 4, 4), dtype=torch.float64)

        # Create the ground truth cost volume (row, col, disp)
        cv_gt = np.full((4, 4, 4), np.nan)

        # disparity -4
        # all nan
        # disparity -3
        cv_gt[:, 3:, 1] = self.sad_cost(left_feature[:, :, :, 3:], right_features[:, :, :, 0:1]).cpu().detach().numpy()
        # disparity -2
        cv_gt[:, 2:, 2] = self.sad_cost(left_feature[:, :, :, 2:], right_features[:, :, :, 0:2]).cpu().detach().numpy()
        # disparity -1
        cv_gt[:, 1:, 3] = self.sad_cost(left_feature[:, :, :, 1:], right_features[:, :, :, 0:3]).cpu().detach().numpy()

        # The minus sign converts the similarity score to a matching cost
        cv_gt *= -1

        acc = AccMcCnnInfer()
        # Because input shape of nn.Conv2d is (Batch_size, Channel, H, W), we add 1 dimensions
        cv = acc.computes_cost_volume_mc_cnn_accurate(left_feature, right_features, -4, -1, self.sad_cost)

        # Check if the calculated cost volume is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_allclose(cv, cv_gt, rtol=1e-05)

    def test_computes_cost_volume_mc_cnn_accurate_positive_disp(self):
        """ "
        Test the computes_cost_volume_mc_cnn_accurate function with positive disparities

        """
        # create left and right features
        left_feature = torch.randn((1, 112, 4, 4), dtype=torch.float64)
        right_features = torch.randn((1, 112, 4, 4), dtype=torch.float64)

        # Create the ground truth cost volume (row, col, disp)
        cv_gt = np.full((4, 4, 4), np.nan)

        # disparity 1
        cv_gt[:, :3, 0] = self.sad_cost(left_feature[:, :, :, :3], right_features[:, :, :, 1:4]).cpu().detach().numpy()
        # disparity 2
        cv_gt[:, :2, 1] = self.sad_cost(left_feature[:, :, :, :2], right_features[:, :, :, 2:4]).cpu().detach().numpy()
        # disparity 3
        cv_gt[:, :1, 2] = self.sad_cost(left_feature[:, :, :, :1], right_features[:, :, :, 3:]).cpu().detach().numpy()
        # disparity 4
        # all nan

        # The minus sign converts the similarity score to a matching cost
        cv_gt *= -1

        acc = AccMcCnnInfer()
        # Because input shape of nn.Conv2d is (Batch_size, Channel, H, W), we add 1 dimensions
        cv = acc.computes_cost_volume_mc_cnn_accurate(left_feature, right_features, 1, 4, self.sad_cost)

        # Check if the calculated cost volume is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_allclose(cv, cv_gt, rtol=1e-05)

    # pylint: disable=invalid-name
    # -> because changing the name here loses the reference to the actual name of the checked function
    def test_MiddleburyGenerator(self):
        """
        test the function MiddleburyGenerator
        """
        # Script use to create images_middlebury and samples_middlebury :
        # pylint: disable=pointless-string-statement
        """
        # shape 1, 2, 13, 13 : 1 exposures, 2 = left and right images
        image_pairs_0 = np.zeros((1, 2, 13, 13))
        # left
        image_pairs_0[0, 0, :, :] = np.tile(np.arange(13), (13, 1))
        # right
        image_pairs_0[0, 1, :, :] = np.tile(np.arange(13), (13, 1)) + 1

        image_pairs_1 = np.zeros((1, 2, 13, 13))
        image_pairs_1[0, 0, :, :] = np.tile(np.arange(13), (13, 1))
        image_pairs_1[0, 1, :, :] = np.tile(np.arange(13), (13, 1)) - 1

        img_file = h5py.File('images_middlebury.hdf5', 'w')
        img_0 = [image_pairs_0]
        grp = img_file.create_group(str(0))
        # 1 illumination
        for light in range(len(img_0)):
            dset = grp.create_dataset(str(light), data=img_0[light])

        img_1 = [image_pairs_1]
        grp = img_file.create_group(str(1))
        for light in range(len(img_1)):
            dset = grp.create_dataset(str(light), data=img_1[light])

        sampl_file = h5py.File('sample_middlebury.hdf5', 'w')
        # disparity of image_pairs_0
        x0 = np.array([[0., 5., 6., 1.]
                       [0., 7., 7., 1.]])
        # disparity of image_pairs_1
        x1 = np.array([[ 1.,  7.,  5., -1.]
                       [ 0.,  0.,  0.,  0.]])
        sampl_file.create_dataset(str(0), data=x0)
        sampl_file.create_dataset(str(1), data=x1)
        """

        # Positive disparity
        cfg = {
            "data_augmentation": False,
            "dataset_neg_low": 1,
            "dataset_neg_high": 1,
            "dataset_pos": 0,
            "augmentation_param": {
                "vertical_disp": 0,
                "scale": 0.8,
                "hscale": 0.8,
                "hshear": 0.1,
                "trans": 0,
                "rotate": 28,
                "brightness": 1.3,
                "contrast": 1.1,
                "d_hscale": 0.9,
                "d_hshear": 0.3,
                "d_vtrans": 1,
                "d_rotate": 3,
                "d_brightness": 0.7,
                "d_contrast": 1.1,
            },
        }

        training_loader = MiddleburyGenerator("tests/sample_middlebury.hdf5", "tests/images_middlebury.hdf5", cfg)
        # Patch of shape 3, 11, 11
        # With the firt dimension = left patch, right positive patch, right negative patch
        patch = training_loader[0]

        x_left_patch = 6
        y_left_patch = 5
        patch_size = 5
        gt_left_patch = self.left_img_0[
            y_left_patch - patch_size : y_left_patch + patch_size + 1,
            x_left_patch - patch_size : x_left_patch + patch_size + 1,
        ]

        # disp = 1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        disp = 1
        x_right_pos_patch = x_left_patch - disp
        y_right_pos_patch = 5
        gt_right_pos_patch = self.right_img_0[
            y_right_pos_patch - patch_size : y_right_pos_patch + patch_size + 1,
            x_right_pos_patch - patch_size : x_right_pos_patch + patch_size + 1,
        ]

        # dataset_neg_low & dataset_neg_high = 1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        dataset_neg = 1
        x_right_neg_patch = x_left_patch - disp + dataset_neg
        y_right_neg_patch = 5
        gt_right_neg_patch = self.right_img_0[
            y_right_neg_patch - patch_size : y_right_neg_patch + patch_size + 1,
            x_right_neg_patch - patch_size : x_right_neg_patch + patch_size + 1,
        ]

        gt_path = np.stack((gt_left_patch, gt_right_pos_patch, gt_right_neg_patch), axis=0)

        # Check if the calculated patch is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_array_equal(patch, gt_path)

        # negative disparity
        patch = training_loader[2]

        x_left_patch = 5
        y_left_patch = 7
        patch_size = 5
        gt_left_patch = self.left_img_0[
            y_left_patch - patch_size : y_left_patch + patch_size + 1,
            x_left_patch - patch_size : x_left_patch + patch_size + 1,
        ]

        # disp = -1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        disp = -1
        x_right_pos_patch = x_left_patch - disp
        y_right_pos_patch = 5
        gt_right_pos_patch = self.right_img_0[
            y_right_pos_patch - patch_size : y_right_pos_patch + patch_size + 1,
            x_right_pos_patch - patch_size : x_right_pos_patch + patch_size + 1,
        ]

        # dataset_neg_low & dataset_neg_high = 1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        dataset_neg = 1
        x_right_neg_patch = x_left_patch - disp + dataset_neg
        y_right_neg_patch = 5
        gt_right_neg_patch = self.right_img_0[
            y_right_neg_patch - patch_size : y_right_neg_patch + patch_size + 1,
            x_right_neg_patch - patch_size : x_right_neg_patch + patch_size + 1,
        ]

        gt_path = np.stack((gt_left_patch, gt_right_pos_patch, gt_right_neg_patch), axis=0)

        # Check if the calculated patch is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_array_equal(patch, gt_path)

    # pylint: disable=invalid-name
    # -> because changing the name here loses the reference to the actual name of the checked function
    def test_DataFusionContestGenerator(self):
        """
        test the function DataFusionContestGenerator
        """
        # pylint: disable=pointless-string-statement
        """
        # Script use to create images_middlebury and samples_middlebury :
        # shape 2, 13, 13 : 2 = left and right images, row, col
        image_pairs_0 = np.zeros((2, 13, 13))
        # left
        image_pairs_0[0, :, :] = np.tile(np.arange(13), (13, 1))
        # right
        image_pairs_0[1, :, :] = np.tile(np.arange(13), (13, 1)) + 1

        image_pairs_1 = np.zeros((2, 13, 13))
        image_pairs_1[0, :, :] = np.tile(np.arange(13), (13, 1))
        image_pairs_1[1, :, :] = np.tile(np.arange(13), (13, 1)) - 1

        img_file = h5py.File('images_dfc.hdf5', 'w')
        img_file.create_dataset(str(0), data=image_pairs_0)
        img_file.create_dataset(str(1), data=image_pairs_1)

        sampl_file = h5py.File('sample_dfc.hdf5', 'w')
        # disparity of image_pairs_0
        x0 = np.array([[0., 5., 6., 1.],
                       [0., 7., 7., 1.]])
        # disparity of image_pairs_1
        x1 = np.array([[ 1.,  7.,  5., -1.],
                       [ 0.,  0.,  0.,  0.]])
        sampl_file.create_dataset(str(0), data=x0)
        sampl_file.create_dataset(str(1), data=x1)
        """
        # Positive disparity
        cfg = {
            "data_augmentation": False,
            "dataset_neg_low": 1,
            "dataset_neg_high": 1,
            "dataset_pos": 0,
            "vertical_disp": 0,
            "augmentation_param": {
                "scale": 0.8,
                "hscale": 0.8,
                "hshear": 0.1,
                "trans": 0,
                "rotate": 28,
                "brightness": 1.3,
                "contrast": 1.1,
                "d_hscale": 0.9,
                "d_hshear": 0.3,
                "d_vtrans": 1,
                "d_rotate": 3,
                "d_brightness": 0.7,
                "d_contrast": 1.1,
            },
        }

        training_loader = DataFusionContestGenerator("tests/sample_dfc.hdf5", "tests/images_dfc.hdf5", cfg)
        # Patch of shape 3, 11, 11
        # With the firt dimension = left patch, right positive patch, right negative patch
        patch = training_loader[0]

        x_left_patch = 6
        y_left_patch = 5
        patch_size = 5
        gt_left_patch = self.left_img_0[
            y_left_patch - patch_size : y_left_patch + patch_size + 1,
            x_left_patch - patch_size : x_left_patch + patch_size + 1,
        ]

        # disp = 1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        disp = 1
        x_right_pos_patch = x_left_patch - disp
        y_right_pos_patch = 5
        gt_right_pos_patch = self.right_img_0[
            y_right_pos_patch - patch_size : y_right_pos_patch + patch_size + 1,
            x_right_pos_patch - patch_size : x_right_pos_patch + patch_size + 1,
        ]

        # dataset_neg_low & dataset_neg_high = 1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        dataset_neg = 1
        x_right_neg_patch = x_left_patch - disp + dataset_neg
        y_right_neg_patch = 5
        gt_right_neg_patch = self.right_img_0[
            y_right_neg_patch - patch_size : y_right_neg_patch + patch_size + 1,
            x_right_neg_patch - patch_size : x_right_neg_patch + patch_size + 1,
        ]
        gt_path = np.stack((gt_left_patch, gt_right_pos_patch, gt_right_neg_patch), axis=0)

        # Check if the calculated patch is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_array_equal(patch, gt_path)

        # negative disparity
        patch = training_loader[2]

        x_left_patch = 5
        y_left_patch = 7
        patch_size = 5
        gt_left_patch = self.left_img_1[
            y_left_patch - patch_size : y_left_patch + patch_size + 1,
            x_left_patch - patch_size : x_left_patch + patch_size + 1,
        ]

        # disp = -1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        disp = -1
        x_right_pos_patch = x_left_patch - disp
        y_right_pos_patch = 7
        gt_right_pos_patch = self.right_img_2[
            y_right_pos_patch - patch_size : y_right_pos_patch + patch_size + 1,
            x_right_pos_patch - patch_size : x_right_pos_patch + patch_size + 1,
        ]

        # dataset_neg_low & dataset_neg_high = 1, with middlebury image convention img_left(x,y) = img_right(x-d,y)
        dataset_neg = 1
        x_right_neg_patch = x_left_patch - disp + dataset_neg
        y_right_neg_patch = 7
        gt_right_neg_patch = self.right_img_2[
            y_right_neg_patch - patch_size : y_right_neg_patch + patch_size + 1,
            x_right_neg_patch - patch_size : x_right_neg_patch + patch_size + 1,
        ]

        gt_path = np.stack((gt_left_patch, gt_right_pos_patch, gt_right_neg_patch), axis=0)

        # Check if the calculated patch is equal to the ground truth (same shape and all elements equals)
        np.testing.assert_array_equal(patch, gt_path)

    def test_accessor_weights(self):
        """
        Tests whether the get_weights function return the accurate path
        """
        device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

        # Load MC-CNN-fast weights trained on Middlebury in the model
        weights_path = get_weights()
        assert "mb" in str(weights_path)
        fast_net = FastMcCnn()
        fast_net.load_state_dict(torch.load(weights_path, map_location=device)["model"])
        fast_net.eval()

        # Load MC-CNN-fast weights trained on DFC in the model
        weights_path = get_weights(training_dataset="dfc")
        assert "data_fusion_contest" in str(weights_path)
        net = FastMcCnn()
        net.load_state_dict(torch.load(weights_path, map_location=device)["model"])
        net.eval()

        # Load MC-CNN-accurate weights trained on Middlebury in the model
        weights_path = get_weights(arch="accurate", training_dataset="middlebury")
        assert "mb" in str(weights_path)
        net = AccMcCnnInfer()
        net.load_state_dict(torch.load(weights_path, map_location=device)["model"])
        net.eval()

        # Load MC-CNN-accurate weights trained on DFC in the model
        weights_path = get_weights(arch="accurate", training_dataset="dfc")
        assert "data_fusion_contest" in str(weights_path)
        net = AccMcCnnInfer()
        net.load_state_dict(torch.load(weights_path, map_location=device)["model"])
        net.eval()


if __name__ == "__main__":
    unittest.main()
